# This is the installation program for Log Scanner.  
#
# To install logscanner, just run this file like this:
#
# > perl Install
#
#

use Cwd;
use File::Copy;
$|=1;

$SIG{INT} = sub {
	print "\n";
	print "\nLog Scanner installation aborted.\n";
	print "\n";
	exit();
};

# First, let's figure out where we live.
$tmphome=$0;
if($tmphome!~/\//) {
	$home=".";
} else {
	$tmphome=~/(.*)\//;
	$home=$1;
}

# We need to make sure all the files are here.
@files=("brains.pl","files.cfg","general.pl","loadconf.pl","logscanner","numeric.scr","pager.pl","required.pl");
@lscanfiles=("Rules.cfg","base_functions.mod","sample_functions.mod");


foreach $file (@files) {
	$fname="$home/$file";
	if(!-f $fname) {
		print "Could not find required file \"$file\" in $home.\n";
		exit();
	} elsif (!-r $fname) {
		print "Could not read required file \"$file\" in $home.\n";
		exit();
	}
}

foreach $file (@lscanfiles) {
	$fname="$home/lscan/$file";
    if(!-f $fname) {
        print "Could not find required file \"$file\" in $home/lscan.\n";
        exit();
    } elsif (!-r $fname) {
        print "Could not read required file \"$file\" in $home/lscan.\n";
        exit(); 
    }
}

# Find version if possible
open(LS,"$home/logscanner");
$version=0;
while(!$version) {
	$buffer=<LS>;
	if ($buffer=~/^\#\s*Log\s*Scanner\s*(.*?)\n$/) {
		$version=$1;
	} elsif ($buffer eq "") {
		$version=-1;
	}
}
close(LS);

# Check to see if we are root
if ($< ne 0) {
	warning("Log Scanner was intended to be installed and run as root");
}

print "\nWelcome to the Log Scanner ";
print "Version $version " if $version ne -1;
print "Install program.\n\n";

warning("This program WILL overwrite your existing Rules.cfg");

$defaultpl="/usr/local/logscanner";
$defaultcl="/etc/lscan";
$perl="/usr/bin/perl";
$allset=0;

print "\nI'm going to ask you some questions to help set up logscanner.\n";
while (!$allset) {

	$perlisgood=0;
	while(!$perlisgood) {
		print "\nWhere does your perl live? (Default: $perl)\n";
		print "> ";
		$newperl=<STDIN>;
		chomp($newperl);

		$newperl=$perl if($newperl=~/^\s*$/);
		$newperl="./$newperl" if ($newperl!~/^\//);

		$perlisgood=1;
		if(!-f $newperl) {
			$perlisgood=0;
			print "I'm sorry, but $newperl doesn't seem to exist.\n";
		} elsif (!-r $newperl) {
			$perlisgood=0;
			print "I'm sorry, but $newperl is not readable to you.\n";
			print "It's kinda important that it be readable. ;)\n";
		} elsif (!-x $newperl) {
			$perlisgood=0;
			print "I'm sorry, but $newperl isn't executable.\n";
			print "Makes it kinda tough to run.  ;)\n";
		} else {
			$perl=$newperl;
		}
	}

	$plisgood=0;
	while(!$plisgood) {
		$tpath="";
		$t2path="";
		$createpl="";
		print "\nWhere would you like to install Log Scanner?\n";
		print "(Default: $defaultpl)\n";
		print "> ";
		$newpl=<STDIN>;
		chomp($newpl);

		$newpl=$defaultpl if ($newpl=~/^\s*$/);
		$newpl="./$newpl" if ($newpl!~/^\// && $newpl!~/^\./);
		
		$plisgood=1;
		if(!-d $newpl) {
# Let's see if any of this path exists. 
			@p=split(/\//,$newpl);
			$i=0;
			$tval="/" if $p[0] eq "";
			$tval=$p[0] if $p[0] ne "";
			if(!-d $tval) {
				print "Um, apparently '$tval' doesn't exist.\n";
				print "I'm going to freak out mildly, then exit.\n";
				sleep 4;
				print "AAAAAAAAAAAGGGGGGGGGGGGHHHHHHHHHHH!!!!!!!!!\n";
				exit();
			}
			for($i=0;$i<=$#p;$i++) {
				$t2path=$tpath;
				$tpath.="/" if ($tpath ne "/" && $p[$i] ne ".");
				$tpath.=$p[$i];

				if(!-d $tpath) {
					last;
				}
			}
			if(!-w $t2path) {
				print "\nUh oh.  $newpl doesn't seem to exist.\n";
				print "The best I could find was $t2path, \n";
				print "and you don't have permission to write to it.\n";
				print "Please try again.\n";
				$plisgood=0;
			} else {
				print "\nHmmm, $ doesn't seem to exist, but I can create it.\n";
				print "Should I? (Y/n) > ";
				$createpl=<STDIN>;
				chomp($createpl);
				$createpl="Y" unless($createpl=~/^n/i);

				$plisgood=0 if $createpl ne "Y";
			}
		} elsif (!-w $newpl) {
			print "\nYou don't have permission to store files in that directory.\n";
			print "Please choose another.\n";
			$plisgood=0;
		}
	}
	$defaultpl=$newpl;

	$clisgood=0;
	while(!$clisgood) {
		$tpath="";
		$t2path="";
		$createcl="";
		print "\nWhere would you like to install Log Scanner?\n";
		print "(Default: $defaultcl)\n";
		print "> ";
		$newcl=<STDIN>;
		chomp($newcl);

		$newcl=$defaultcl if ($newcl=~/^\s*$/);
		$newcl="./$newcl" if ($newcl!~/^\// && $newcl!~/^\./);
		
		$clisgood=1;
		if(!-d $newcl) {
# Let's see if any of this path exists. 
			@p=split(/\//,$newcl);
			$i=0;
			$tval="/" if $p[0] eq "";
			$tval=$p[0] if $p[0] ne "";
			if(!-d $tval) {
				print "Um, apparently '$tval' doesn't exist.\n";
				print "I'm going to freak out mildly, then exit.\n";
				sleep 4;
				print "AAAAAAAAAAAGGGGGGGGGGGGHHHHHHHHHHH!!!!!!!!!\n";
				exit();
			}
			for($i=0;$i<=$#p;$i++) {
				$t2path=$tpath;
				$tpath.="/" if ($tpath ne "/" && $p[$i] ne ".");
				$tpath.=$p[$i];

				if(!-d $tpath) {
					last;
				}
			}
			if(!-w $t2path) {
				print "\nUh oh.  $newcl doesn't seem to exist.\n";
				print "The best I could find was $t2path, \n";
				print "and you don't have permission to write to it.\n";
				print "Please try again.\n";
				$clisgood=0;
			} else {
				print "\nHmmm, $ doesn't seem to exist, but I can create it.\n";
				print "Should I? (Y/n) > ";
				$createcl=<STDIN>;
				chomp($createcl);
				$createcl="Y" unless($createcl=~/^n/i);

				$clisgood=0 if $createcl ne "Y";
			}
		} elsif (!-w $newcl) {
			print "\nYou don't have permission to store files in that directory.\n";
			print "Please choose another.\n";
			$clisgood=0;
		}
	}
	$defaultcl=$newcl;


	$cwd=cwd();
	if($perl=~/^\./) {
		$perl=~s/^\./$cwd/;
	}

	if($defaultpl=~/^\./) {
		$defaultpl=~s/^\./$cwd/;
	}

	if($defaultcl=~/^\./) {
		$defaultcl=~s/^\./$cwd/;
	}

	print "You have chosen: \n";
	print "\tPerl: $perl\n";
	print "\tProgram files: $defaultpl\n";
	print "\tConfig files: $defaultcl\n";
	print "\n";
	print "Are these correct?\n";
	print "(Y/n) > ";

	$allgood=<STDIN>;
	$allset=1 unless $allgood=~/^n/i;
}


# First off, we need to replace the perl lines if necessary
if($perl ne "/usr/bin/perl") {
	print "Replacing perl lines.";
	foreach $file (@files) {
		print ".";
		$f="$home/$file";
		$tmpfname=$f.".tmp";
		open (FH,$f);
		open (TFH,">$tmpfname");
		print TFH "#$perl\n";
		$buffer=<FH>;
		while($buffer=<FH>) {print TFH $buffer;}
		close(TFH);
		close(FH);
		unlink($f);
		rename($tmpfname,$f);
	}
	foreach $file (@lscanfiles) {
        print ".";
        $f="$home/lscan/$file";
        $tmpfname=$f.".tmp";
        open (FH,$f);
        open (TFH,">$tmpfname");
        print TFH "#$perl\n";
        $buffer=<FH>;
        while($buffer=<FH>) {print TFH $buffer;}
        close(TFH);
        close(FH);
        unlink($f);
        rename($tmpfname,$f);
	}
	print "done\n";
}

# Now we have to replace the config directory
$f="$home/files.cfg";
$tf="$home/files.cfg.tmp";
if($defaultcl ne "/etc/lscan") {
	print "Setting the configuration directory.";
	open(F,$f);
	print ".";
	open(TF,">$tf");
	print ".";

	while($buffer=<F>) {
		print ".";
		if($buffer=~/^\s*\$conf_dir/) {
			chomp($buffer);
			print TF "\$conf_dir = \"$defaultcl\";\n";
		} else {
			print TF $buffer;
		}
	}

	close(TF);
	print ".";
	close(F);
	print ".";
	unlink($f);
	rename($tf,$f);

	print "done\n";
}

# Time to make the directories...
if($createpl eq "Y") {
	print "Making program directory.";

	@s=split(/\//,$defaultpl);
	$tp="";
	for($i=1;$s[$i] ne "";$i++) {
		print ".";
		$tp.="/$s[$i]";
		mkdir($tp,0700) if(!-d $tp); 
	}
	print "done\n";
}

if($createcl eq "Y") {
	print "Making program directory.";

	@s=split(/\//,$defaultcl);
	$tp="";
	for($i=1;$s[$i] ne "";$i++) {
		print ".";
		$tp.="/$s[$i]";
		mkdir($tp,0700) if(!-d $tp); 
	}
	print "done\n";
}

# Time to install it!
print "Copying files";
foreach $file (@files) {
	print ".";
	$f="$home/$file";
	$nf="$defaultpl/$file";
	copy($f,$nf);
}

foreach $file (@lscanfiles) {
	print ".";
	$f="$home/lscan/$file";
	$nf="$defaultcl/$file";
	copy($f,$nf);
}
print "done\n";

chmod 0700,"$defaultpl/logscanner";

print "\n\nLog Scanner installation Complete!\n\n";


##### Quasi useful subroutine(s) #####
sub warning {
	print "*"x(length($_[0])+4);
	print "\n";

	print "* ";
	$distance = (length($_[0])-8)/2;
	print " "x(int($distance));
	print "Warning!";
	print " "x(int($distance+.5));
	print " *\n";

	print "* $_[0] *\n";
	
	print "*"x(length($_[0])+4);
	print "\n";

}
